// ==============================================================
//                 ORBITER MODULE: starlink_odd
//                  
//
// starlink_odd.cpp
// Control module for starlink_odd vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK40.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include <math.h>

// ==============================================================
// Some vessel parameters
// ==============================================================

const double ISP = 11000;
const double MAINTH = 0.01;
const double ARRAY_SPEED = 0.05;
const double FUEL_MASS = 50;
const double EMP_MASS = 227;

#define YELLOW RGB(255, 255, 80)
#define GREEN RGB(0, 225, 0)

static TOUCHDOWNVTX tdvtx_geardown[3] = {
	{_V( 0,-1, -1.6), 6e4, 2e3, 3.0, 0.2},
	{_V(-0.5, 0.866, -1.6), 6e4, 2e3, 3.0, 0.2},
	{_V( 0.5, 0.866, -1.6), 6e4, 2e3, 3.0, 0.2}
};




// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = 0.0;
	*cd = 0.95 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}

void HLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = 0.0;
	*cd = 0.95 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}


class starlink_odd: public VESSEL3 {
public:
	
	starlink_odd (OBJHANDLE hVessel, int flightmodel);
	~starlink_odd ();

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	

private:
	void DefineAnimations (void);
	void DoKillRot(void);
	void DoPrograde(void);
	void DoRetrograde(void);
	void DoOrbitNormal(void);
	void DoOrbitAntiNormal(void);
	void DoNadirPointing(void);

	MGROUP_TRANSFORM *MgrpTran[6];
	UINT anim_array;
	int MyID;
	THRUSTER_HANDLE th_main;
	PROPELLANT_HANDLE hProp;
	ANIMATIONCOMPONENT_HANDLE Section0, Section1, Section2, Section3, Section4;
	ATTACHMENTHANDLE hAttach;
	double array_proc, deploy_timer;
	int array_status, deploy_status, nad_ptg_sw;


};

starlink_odd::starlink_odd (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{

}

starlink_odd::~starlink_odd ()
{
int i;
for (i = 0; i < 6; i++)
	{
	delete MgrpTran[i];
	}
}

// Define animation sequences for moving parts
// --------------------------------------------------------------
void starlink_odd::DefineAnimations (void)
{
	// array
	anim_array = CreateAnimation (0);

	static UINT ArrayGrp0[1] = {0};
	MgrpTran[0] = new MGROUP_ROTATE(0, ArrayGrp0, 1, _V(-0.83,0.102,0), _V(0,0,1), (float)(90*RAD));
	Section0 = AddAnimationComponent (anim_array, 0.0, 1, MgrpTran[0]);

	static UINT ArrayGrp1[1] = {1};
	MgrpTran[1] = new MGROUP_ROTATE(0, ArrayGrp1, 1, _V(0.571,0.106,0), _V(0,0,1), (float)(-180*RAD));
	Section1 = AddAnimationComponent (anim_array, 0.0, 1, MgrpTran[1], Section0);

	static UINT ArrayGrp2[1] = {2};
	MgrpTran[2] = new MGROUP_ROTATE(0, ArrayGrp2, 1, _V(-0.571,0.110,0), _V(0,0,1), (float)(180*RAD));
	Section2 = AddAnimationComponent (anim_array, 0.0, 1, MgrpTran[2], Section1);

	static UINT ArrayGrp3[1] = {3};
	MgrpTran[3] = new MGROUP_ROTATE(0, ArrayGrp3, 1, _V(0.571,0.115,0), _V(0,0,1), (float)(-180*RAD));
	Section3 = AddAnimationComponent (anim_array, 0.0, 1, MgrpTran[3], Section2);

	static UINT ArrayGrp4[1] = {4};
	MgrpTran[4] = new MGROUP_ROTATE(0, ArrayGrp4, 1, _V(-0.571,0.119,0), _V(0,0,1), (float)(180*RAD));
	Section4 = AddAnimationComponent (anim_array, 0.0, 1, MgrpTran[4], Section3);

	static UINT ArrayGrp5[1] = {5};
	MgrpTran[5] = new MGROUP_ROTATE(0, ArrayGrp5, 1, _V(0.571,0.123,0), _V(0,0,1), (float)(-180*RAD));
	AddAnimationComponent (anim_array, 0.0, 1, MgrpTran[5], Section4);


}



// ==============================================================
// starlink_odd class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void starlink_odd::clbkSetClassCaps (FILEHANDLE cfg)
{
	array_proc = 0.0;
	array_status = 0;
	deploy_status = 0;
	deploy_timer = 0.0;
	nad_ptg_sw = 0;
	
	DefineAnimations ();
	// physical specs
	SetCW (1, 1, 1, 1);
	SetRotDrag (_V(1,1,1));
	SetPMI (_V(2,2,1));
	SetCrossSections (_V(1,1,1));
	SetSize (2);
	SetEmptyMass (EMP_MASS);
	SetMeshVisibilityMode ((AddMesh ("starlink/starlink_odd")), MESHVIS_ALWAYS);
	
	SetCameraOffset (_V(0,1.5,-1.8));
	//SetTouchdownPoints (_V(0.1,0.8,-1.8), _V(0.512,-0.853,-1.8), _V(-0.512,-0.853,-1.8));
	SetTouchdownPoints ( tdvtx_geardown, 3);
	//SetDockParams (_V(0,0,-2), _V(0,0,-1), _V(0,1,0));

	// propellant resources

	hProp = CreatePropellantResource (FUEL_MASS, FUEL_MASS);

	
	// ***************** thruster definitions *******************
	th_main = CreateThruster (_V( 0,0,-1.9), _V(0,0, 1), MAINTH, hProp, ISP);
	
	

	
	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("starlink/exhaust");
	AddExhaust (th_main, 2, 0.04, _V(-0.612, 0,-1.96), _V(0,0,-1),(tex_main));
	AddExhaust (th_main, 2, 0.04, _V( 0.612, 0,-1.96), _V(0,0,-1),(tex_main));

	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);



	///////////////////////////////////////////////////////
	/////AirFoils

	CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 0, 1, 1, 1);
	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0, 1, 1, 1);

	//////////////////// attach //////////////////////////

	hAttach = CreateAttachment (true, _V( 0, 0, 0), _V( 0,-1,0), _V(-1,0,0), "EVEN");

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new starlink_odd (hvessel, flightmodel);
}




// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (starlink_odd*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void starlink_odd::clbkPostCreation ()
{
	
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	ReplaceStockSound(MyID,"Sound\\Vessel\\attsustain.wav",	REPLACE_MAIN_THRUST);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);
	
}
void starlink_odd::clbkSaveState (FILEHANDLE scn)
{

	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters

	sprintf (cbuf, "%d %0.4f", array_status, array_proc);
	oapiWriteScenario_string (scn, "SOLP", cbuf);

	sprintf (cbuf, "%d %d %0.4f", nad_ptg_sw, deploy_status,  deploy_timer);
	oapiWriteScenario_string (scn, "DPLY", cbuf);

	
}
void starlink_odd::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "SOLP", 4))
		{
            sscanf (line+4, "%d%lf", &array_status, &array_proc);
		}
		else if (!strnicmp (line, "DPLY", 4))
		{
            sscanf (line+4, "%d%d%lf", &nad_ptg_sw, &deploy_status, &deploy_timer);
		}
		else
		{
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
	SetAnimation (anim_array, array_proc);
	
	
}






int starlink_odd::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{

//bool cbSelectTarget(void *id, char *str, void *data);

  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) // +shift
	{
	  switch (key) 
	  {
		
	  }
	  return true;
	} 
  else if (KEYMOD_CONTROL (kstate)) // +ctrl
	{
  
	}
  else // unmodified keys
	{ 
    switch (key) 
		{
	case OAPI_KEY_G: 
		if (array_status  == 0)array_status = 1;
		return 1;

		case OAPI_KEY_K: 
		if(nad_ptg_sw == 0)
		{
			nad_ptg_sw = 1;
			DeactivateNavmode (NAVMODE_KILLROT);
			DeactivateNavmode (NAVMODE_PROGRADE);
			DeactivateNavmode (NAVMODE_RETROGRADE);
			DeactivateNavmode (NAVMODE_NORMAL);
			DeactivateNavmode (NAVMODE_ANTINORMAL);
		}
		else nad_ptg_sw = 0;
		return 1;

		//case OAPI_KEY_J: 
		//
		//return 1;
		}
	}
  
 return 0;
 }


int starlink_odd::clbkConsumeDirectKey(char *kstate)
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*2*dt;

//if (KEYMOD_SHIFT (kstate))
{
if (KEYDOWN (kstate, OAPI_KEY_NUMPAD2))
		{
		if (avel.x < RAD*10)
			{
			avel.x = avel.x+drot;
			if (avel.x > RAD*10) avel.x = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD8))
		{
		if (avel.x > -RAD*10)
			{
			avel.x = avel.x-drot;
			if (avel.x < -RAD*10) avel.x = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD1))
		{
		if (avel.y < RAD*10)
			{
			avel.y = avel.y+drot;
			if (avel.y > RAD*10) avel.y = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD3))
		{
		if (avel.y > -RAD*10)
			{
			avel.y = avel.y-drot;
			if (avel.y < -RAD*10) avel.y = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD6))
		{
		if (avel.z < RAD*10)
			{			
			avel.z = avel.z+drot;
			if (avel.z > RAD*10) avel.z = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD4))
		{
		if (avel.z > -RAD*10)
			{
			avel.z = avel.z-drot;
			if (avel.z < -RAD*10) avel.z = -RAD*10;
			}		
		}

		SetAngularVel(avel);
	}
return 0;
}





bool starlink_odd::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{

	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	//int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	//int w3 = (int)(232*d);
	//int w4 = (int)(248*d);
	//int w5 = (int)(264*d);
	//int w6 = (int)(280*d);
	//int w7 = (int)(296*d);
	//int w8 = (int)(312*d);
	//int w9 = (int)(328*d);
	//int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);

	{
	skp->SetTextColor(0x0066FF66);
	double m1 = GetEmptyMass();
	double m2 = GetPropellantMass(hProp);
	double mass_fraction = ((m1+m2)/(m1));
	double dv = (ISP*(log(mass_fraction)));
	char cbuf[256];
	sprintf(cbuf,"Available dV %0.2f m/s", dv);
	skp->Text ((10+roxl), (wc+royl), cbuf,strlen(cbuf));
	}


	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"NumPad = Reaction Wheels/Torquers");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	if(nad_ptg_sw == 0)sprintf(abuf,"K = Nadir Pointing");
	else sprintf(abuf,"K = Nadir Pointing ON");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	if(array_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"G = Deploy Array");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	return true;
}


void starlink_odd::clbkPreStep(double SimT, double SimDT, double MJD)
{
	if(deploy_status == 0)
	{
		if(GetAttachmentStatus(hAttach) == NULL) deploy_status = 1;
	}
	else if(deploy_status == 1)
	{
		deploy_timer = deploy_timer + SimDT;
		if(deploy_timer > 2000)
		{
			deploy_status = 2;
			array_status = 1;
			nad_ptg_sw = 1;
			DeactivateNavmode (NAVMODE_KILLROT);
			DeactivateNavmode (NAVMODE_PROGRADE);
			DeactivateNavmode (NAVMODE_RETROGRADE);
			DeactivateNavmode (NAVMODE_NORMAL);
			DeactivateNavmode (NAVMODE_ANTINORMAL);
		}
	}

	
	if(GetNavmodeState (NAVMODE_KILLROT) == true)
	{
		DoKillRot();
		nad_ptg_sw = 0;
	}
	else if(GetNavmodeState (NAVMODE_PROGRADE) == true)
	{
		DoPrograde();
		nad_ptg_sw = 0;
	}
	else if(GetNavmodeState (NAVMODE_RETROGRADE) == true)
	{
		DoRetrograde();
		nad_ptg_sw = 0;
	}
	else if(GetNavmodeState (NAVMODE_NORMAL) == true)
	{
		DoOrbitNormal();
		nad_ptg_sw = 0;
	}
	else if(GetNavmodeState (NAVMODE_ANTINORMAL) == true)
	{
		DoOrbitAntiNormal();
		nad_ptg_sw = 0;
	}
	else if (nad_ptg_sw == 1)DoNadirPointing();
	





	///////// animations //////////////////////////////////

	// deploy solp
	if (array_status == 0) array_proc = 0;
	else if (array_status == 1) 
	{
		double da = SimDT * ARRAY_SPEED;
		array_proc = array_proc + da;
		if (array_proc > 1) 
		{
			array_proc = 1;
			array_status = 2;
		}
	}
	else if (array_status == 2) array_proc = 1;
	SetAnimation (anim_array, array_proc);
	
}


//void starlink_odd::clbkPostStep (double simt, double simdt, double mjd)
//{
//
//}

void starlink_odd::DoKillRot()
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*2*dt;

		if (drot > abs(avel.x)) avel.x = 0;
		else avel.x = avel.x + (drot * ((abs(avel.x))/-avel.x));
		if (drot > abs(avel.y)) avel.y = 0;
		else avel.y = avel.y + (drot * ((abs(avel.y))/-avel.y));
		if (drot > abs(avel.z)) avel.z = 0;
		else avel.z = avel.z + (drot * ((abs(avel.z))/-avel.z));

		SetAngularVel (avel);
		//if (avel.x == 0 && avel.y == 0 && avel.z == 0) kilrot_sw = 0;
}

void starlink_odd::DoPrograde()
{
VESSELSTATUS vs;
GetStatus(vs);

VECTOR3 pos_unit = vs.rpos;
normalise(pos_unit);
VECTOR3 vel_unit = vs.rvel;
normalise(vel_unit);
VECTOR3 nml_unit = crossp(vel_unit, pos_unit);
normalise(nml_unit);

pos_unit = crossp(nml_unit, vel_unit);
normalise(pos_unit);

double gamma = atan2( pos_unit.x, nml_unit.x);
double beta = -1*(asin(vel_unit.x));
double alpha = atan2(vel_unit.y, vel_unit.z);

vs.arot = _V(alpha, beta, (PI/2)-gamma);
vs.vrot = _V(0,0,0);

DefSetState(&vs);

}

void starlink_odd::DoRetrograde()
{
VESSELSTATUS vs;
GetStatus(vs);

VECTOR3 pos_unit = -vs.rpos;
normalise(pos_unit);
VECTOR3 vel_unit = -vs.rvel;
normalise(vel_unit);
VECTOR3 nml_unit = crossp(vel_unit, pos_unit);
normalise(nml_unit);

pos_unit = crossp(nml_unit, vel_unit);
normalise(pos_unit);

double gamma = atan2( pos_unit.x, nml_unit.x);
double beta = -1*(asin(vel_unit.x));
double alpha = atan2(vel_unit.y, vel_unit.z);

vs.arot = _V(alpha, beta, (PI/2)-gamma);
vs.vrot = _V(0,0,0);

DefSetState(&vs);
}

void starlink_odd::DoOrbitNormal()
{
VESSELSTATUS vs;
GetStatus(vs);

VECTOR3 pos_unit = vs.rpos;
normalise(pos_unit);
VECTOR3 vel_unit = vs.rvel;
normalise(vel_unit);
VECTOR3 nml_unit = crossp(vel_unit, pos_unit);
normalise(nml_unit);

pos_unit = crossp(nml_unit, vel_unit);
normalise(pos_unit);

VECTOR3 xright = pos_unit;
VECTOR3 yup = -vel_unit;
VECTOR3 zforward = nml_unit;


double gamma = atan2( xright.x, yup.x);
double beta = -1*(asin(zforward.x));
double alpha = atan2(zforward.y, zforward.z);

vs.arot = _V(alpha, beta, (PI/2)-gamma);
vs.vrot = _V(0,0,0);

DefSetState(&vs);
}

void starlink_odd::DoOrbitAntiNormal()
{
VESSELSTATUS vs;
GetStatus(vs);

VECTOR3 pos_unit = vs.rpos;
normalise(pos_unit);
VECTOR3 vel_unit = vs.rvel;
normalise(vel_unit);
VECTOR3 nml_unit = crossp(vel_unit, pos_unit);
normalise(nml_unit);

pos_unit = crossp(nml_unit, vel_unit);
normalise(pos_unit);

VECTOR3 xright = pos_unit;
VECTOR3 yup = vel_unit;
VECTOR3 zforward = -nml_unit;


double gamma = atan2( xright.x, yup.x);
double beta = -1*(asin(zforward.x));
double alpha = atan2(zforward.y, zforward.z);

vs.arot = _V(alpha, beta, (PI/2)-gamma);
vs.vrot = _V(0,0,0);

DefSetState(&vs);
}

void starlink_odd::DoNadirPointing()
{
	VECTOR3 sun_unit;
	GetGlobalPos(sun_unit);
	sun_unit = -sun_unit;
	normalise(sun_unit);

	VESSELSTATUS vs;
	GetStatus(vs);

	VECTOR3 pos_unit = vs.rpos;
	normalise(pos_unit);

	VECTOR3 nml_unit = crossp(sun_unit, pos_unit);
	normalise(nml_unit);

	sun_unit = crossp(nml_unit, pos_unit);
	normalise(sun_unit);

	VECTOR3 xright = -sun_unit;
	VECTOR3 yup = pos_unit;
	VECTOR3 zforward = nml_unit;


double gamma = atan2( xright.x, yup.x);
double beta = -1*(asin(zforward.x));
double alpha = atan2(zforward.y, zforward.z);

vs.arot = _V(alpha, beta, (PI/2)-gamma);
vs.vrot = _V(0,0,0);

DefSetState(&vs);

}