// ==============================================================
//                 ORBITER MODULE: starlink_stack
//                  
//
// starlink_stack.cpp
// Control module for starlink_stack vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK40.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include <math.h>

// ==============================================================
// Some vessel parameters
// ==============================================================


const double EMP_MASS = 16620;

static TOUCHDOWNVTX tdvtx_geardown[3] = {
	{_V( 0,-1, -1.6), 6e4, 2e3, 3.0, 0.2},
	{_V(-0.5, 0.866, -1.6), 6e4, 2e3, 3.0, 0.2},
	{_V( 0.5, 0.866, -1.6), 6e4, 2e3, 3.0, 0.2}
};




// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = 0.0;
	*cd = 0.95 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}

void HLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = 0.0;
	*cd = 0.95 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}


class starlink_stack: public VESSEL3 {
public:
	
	starlink_stack (OBJHANDLE hVessel, int flightmodel);
	~starlink_stack ();

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	
	//int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//int clbkConsumeDirectKey(char *kstate);
	

private:
	
	ATTACHMENTHANDLE hAttach[60];
	ATTACHMENTHANDLE hStageAtt;
	int deploy_status;
	int MyID;

};

starlink_stack::starlink_stack (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{

}

starlink_stack::~starlink_stack ()
{

}



// ==============================================================
// starlink_stack class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void starlink_stack::clbkSetClassCaps (FILEHANDLE cfg)
{
	
	deploy_status = 0;
	// physical specs
	SetCW (1, 1, 1, 1);
	SetRotDrag (_V(1,1,1));
	SetPMI (_V(1,1,2));
	SetCrossSections (_V(1,1,1));
	SetSize (5);
	SetEmptyMass (EMP_MASS);
	SetCameraOffset (_V(0,1.5,-1.8));
	SetTouchdownPoints ( tdvtx_geardown, 3);

	///////////////////////////////////////////////////////
	/////AirFoils

	CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 0, 1, 1, 1);
	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0, 1, 1, 1);

	//////////////////// attach //////////////////////////

	double top = 7.2 + 0.57 - 0.24;
	double gap = 0.24;


	hStageAtt = CreateAttachment (true, _V( 0, 0, 0), _V( 0,0,-1), _V(0,1,0), "F9");

	char cbuf[256];
	int i;
	for (i = 0; i < 30; i++)
	{
		int i_even = i*2; // 0 - 58
		double z_offset = top - (i_even * gap * 0.5);
		double x_offset = 0.938;
		sprintf (cbuf, "SL%d", i_even);
		hAttach[i_even] = CreateAttachment (false, _V( x_offset, 0, z_offset), _V( 0,0,1), _V(-1,0,0), cbuf);

		int i_odd = i_even + 1; // 1 - 59
		sprintf (cbuf, "SL%d", i_odd);
		hAttach[i_odd] = CreateAttachment (false, _V( -x_offset, 0, z_offset), _V( 0,0,1), _V(-1,0,0), cbuf);

	}
	
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new starlink_stack (hvessel, flightmodel);
}




// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (starlink_stack*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void starlink_stack::clbkPostCreation ()
{
	
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);
	
}
void starlink_stack::clbkSaveState (FILEHANDLE scn)
{

	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters

	sprintf (cbuf, "%d", deploy_status);
	oapiWriteScenario_string (scn, "DPLY", cbuf);

	
}
void starlink_stack::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "DPLY", 4))
		{
            sscanf (line+4, "%d", &deploy_status);
		}
		else
		{
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	

	
	
}






//int starlink_stack::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
//{
//
//  if (!down) return 0; // only process keydown events
//
//  if (KEYMOD_SHIFT (kstate)) // +shift
//	{
//	  switch (key) 
//	  {
//		
//	  }
//	  return true;
//	} 
//  else if (KEYMOD_CONTROL (kstate)) // +ctrl
//	{
//  
//	}
//  else // unmodified keys
//	{ 
//    switch (key) 
//		{
//	/*case OAPI_KEY_G: 
//		
//		return 1;*/
//
//		//case OAPI_KEY_K: 
//		//
//		//return 1;
//
//		//case OAPI_KEY_J: 
//		//
//		//return 1;
//		}
//	}
//  
// return 0;
// }



void starlink_stack::clbkPreStep(double SimT, double SimDT, double MJD)
{
	

	if(deploy_status == 0)
	{
		if(GetAttachmentStatus(hStageAtt) == NULL)
		{
		int i;
		for (i = 0; i < 60; i++)
			{
				double vel_off = 0.59 - (i * 0.01) + (oapiRand() * 0.005);
				DetachChild(hAttach[i], vel_off);
			}
		deploy_status = 1;
		}
	}
	else if(deploy_status == 1) oapiDeleteVessel(GetHandle());

}


//void starlink_stack::clbkPostStep (double simt, double simdt, double mjd)
//{
//
//}

